/**************************************************************
Copyright (C) 2006  Oli Laruelle

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
See the file COPYING for more information
**************************************************************/

/**
* @author: Oli / yesyesnono.co.uk
* simple example processing sketch on how to extract xml data generated by kismet
* and how to visualise some of the value in a simple fashion.
*
**/

import processing.xml.*;

XMLElement xml;

// ARRAY VALUES INDEX
int ESSID_ID      = 0;
int TIME_ID       = 1;
int BSSID_ID      = 2;
int CHANNEL_ID    = 3;
int ENCRYPTION_ID = 4;
int MAXRATE_ID    = 5;


float totalJourneyTime;

float prevX = 0.0f;
float prevY = 0.0f;
float posYoffset = 0.0f;
int incrY = 10;

String UK_WAPPING_BIKE = "Kismet-Aug-05-2007-1.xml";
String dataToLoad = UK_WAPPING_BIKE;

PFont clean;


void setup()
{  
  size(4024,600);
  smooth();
  
  clean = loadFont("sansserif-9.vlw");  
  textFont(clean, 9);

  totalJourneyTime = 0;  
  xml = new XMLElement(this, dataToLoad);
  
  // draw our visualisation
  smooth();
  background(0);
  parseXML();
  
}


void parseXML()
{
  
  // get our xml nodes
  XMLElement nodes = xml.getChild(0); 
  XMLElement[] nodesData = xml.getChildren();
  
  // check how long the journey has been
  float startJourneyTime =  getSeconds(nodesData[0].getStringAttribute("first-time"));
  float endJourneyTime = getSeconds(nodesData[nodesData.length-1].getStringAttribute("first-time"));
  totalJourneyTime = endJourneyTime - startJourneyTime;
  println("totalJourneyTime : "+totalJourneyTime);
  
  // loop through our xml nodes
  for (int i = 0; i < nodesData.length; i++) 
  {
    String essid = nodesData[i].getChild(0).getContent();
    String time = nodesData[i].getStringAttribute("first-time");
    String bssid = nodesData[i].getChild(1).getContent();
    String channel = nodesData[i].getChild(2).getContent();
    String maxrate = nodesData[i].getChild(3).getContent();
    String encryption = nodesData[i].getChild(5).getContent();    //note : some have apparently several encryption mode in the xml format
    //println("essid:"+essid+"  time:"+time+" bssid:"+bssid+" channel:"+channel+" encryption:"+encryption+" maxrate:"+maxrate);
    float relativeJourneyTime = getSeconds(time)-startJourneyTime;
    
    // and represent our nodes one by one
    doEssid(i, essid, relativeJourneyTime, bssid, channel, encryption, maxrate);
  }
  
  // few stats
  println("M0000oogaaaaaaaaaaaa! "+totalJourneyTime/60+" minutes to get "+nodesData.length+" wireless nodes recorded!");
  
  // save the output
  save("looknize.png");
  println("_______ img saved! _______");
}


// draw individual node
void doEssid(int id, String inEssid, float inTime, String inBssid, String inChannel, String inEncryption, String inMaxrate)
{
   
  float posY = (height)/2;
  float posX = (inTime*(width-200))/totalJourneyTime+100;  
  
  // to avoid essid text overlapping
  // we go down or up one step at each node recoding
  posYoffset -= incrY;
  
  // we reached one top or bottom boundary 
  // step the other way round from now on
  if(abs(posYoffset) > height/2-100) incrY = incrY*-1;
  
  
  // draw continuous red line trhough our nodes
  stroke(255,0,0);
  if(posX > 100) line(prevX, prevY, posX, posY+posYoffset);
  
  
  // Define size of circle around node
  // according to its encryption type

  int ellipseWidth = 5;
  
  if(inEncryption.equals("WEP"))
  {
    stroke(255,0,0);
    ellipseWidth = 5;
  }else if(inEncryption.equals("WPA"))
  {
    stroke(0,0,255);
    ellipseWidth = 15;
  }else if(inEncryption.equals("None") || trim(inEncryption).equals("") || trim(inEncryption).equals("0"))
  {
   // whooohoo ! no encryption here
   stroke(200,255,0);
   ellipseWidth = 40;
  }else{
    // have we missed one encryption type ?
    println("some encryption colors aren't mapped like this one dude : "+inEncryption); 
  }
  
  noFill();
  
  // circle around our node
  ellipse(posX,posY+posYoffset, ellipseWidth, ellipseWidth);
  ellipse(posX,posY+posYoffset, 2, 2);
  
  // Text
  pushMatrix();
  translate(posX, posY);
  
  stroke(255);
  line(0,0+2, 0, -4);
  
  noStroke();
  fill(255,100);
  text(inEssid.toLowerCase(), 7,posYoffset);
  
  popMatrix();
    
  prevX = posX;
  prevY = posY+posYoffset;
}



// get time as hh:mm:ss return int of total seconds
int getSeconds(String inTimeStamp){  
  inTimeStamp = splitTokens(inTimeStamp)[3];
  int h =  int(split(inTimeStamp,":")[0])*3600;
  int m =  int(split(inTimeStamp,":")[1])*60;
  int s =  int(split(inTimeStamp,":")[2]);
  int totalSec = h+m+s;
  return totalSec;
}
